import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import deepEqual from 'fast-deep-equal/react.js';
import { Check, CircleSlash, Hourglass, LoaderCircle, X } from "lucide-react";
import { useEffect, useRef } from "react";
import { getBorderStyles, getIconStyles, toValidDOMId } from "../lib/testHelpers";
import { useTestContext } from "./TestContext";
import { useTestGroupContext } from "./TestGroupContext";
const getTestId = (testTitle, groupTitle) => {
    if (!groupTitle) {
        return testTitle;
    }
    return `${groupTitle} - ${testTitle}`;
};
export const TestStateIcon = ({ state }) => {
    const iconClassName = "rbt-icon " + getIconStyles(state);
    if (state === "Pending") {
        return _jsx(Hourglass, { className: `rbt-hourglass ${iconClassName}` });
    }
    else if (state === "Running") {
        return _jsx(LoaderCircle, { className: `rbt-spin-animation ${iconClassName}` });
    }
    else if (state === "Fail") {
        return _jsx(X, { className: iconClassName });
    }
    else if (state === "Success") {
        return _jsx(Check, { className: iconClassName });
    }
    else if (state === "Skipped") {
        return _jsx(CircleSlash, { className: `rbt-skipped ${iconClassName}` });
    }
    return null;
};
export const TitleAndStateDefault = ({ currentTestState, title, ...props }) => {
    return (_jsxs("div", { className: "rbt-test-title-and-result", ...props, children: [_jsx(TestStateIcon, { state: currentTestState }), _jsx("div", { className: "rbt-test-title", children: title })] }));
};
const checkIfTestPropsChanged = (prevProps, props) => {
    const previousFn = prevProps?.fn;
    const currentFn = props.fn;
    const fnChanged = previousFn?.toString() !== currentFn.toString();
    const skipChanged = prevProps?.skip !== props.skip;
    const onlyChanged = prevProps?.only !== props.only;
    const titleChanged = prevProps?.title !== props.title;
    const childrenChanged = !deepEqual(prevProps?.children, props.children);
    return fnChanged || skipChanged || onlyChanged || titleChanged || childrenChanged;
};
export const Test = (props) => {
    const { title, fn, skip = false, only = false, TitleAndState = TitleAndStateDefault, children, ...propsRest } = props;
    const { tests, registerTest, reRunTests, reRunCount, containerId } = useTestContext();
    const groupContext = useTestGroupContext();
    // Refs go here.
    const prevPropsRef = useRef(props);
    const lastReRunCount = useRef(-1);
    const didInit = useRef(false);
    // Get current test's Ids.
    const testId = getTestId(title, groupContext?.groupTitle || "");
    const DOMId = toValidDOMId(testId);
    // Get current test state.
    const currentTest = tests[testId];
    const currentTestState = currentTest?.state || "Pending";
    const hasChildren = !!children;
    const shouldRenderChildren = currentTestState === "Running" ||
        currentTestState === "Success" ||
        currentTestState === "Fail";
    const classNames = "rbt-test " + getBorderStyles(currentTestState);
    const propsHaveChanged = checkIfTestPropsChanged(prevPropsRef.current, props);
    useEffect(() => {
        // Skip effect if:
        // 1. The test has already been registered.
        // 2. The props haven't changed.
        // 3. reRunTests has already been called for this reRunCount.
        if (didInit.current && !propsHaveChanged && lastReRunCount.current === reRunCount) {
            return;
        }
        if (propsHaveChanged) {
            reRunTests();
        }
        const test = {
            title,
            groupTitle: groupContext?.groupTitle,
            id: testId,
            fn,
            skip,
            only,
            state: "Pending",
        };
        registerTest(test);
        prevPropsRef.current = props;
        lastReRunCount.current = reRunCount;
        didInit.current = true;
    }, [propsHaveChanged, reRunCount]);
    return _jsxs("div", { className: classNames, "data-test": containerId, id: DOMId, ...propsRest, children: [_jsx(TitleAndState, { currentTestState: currentTestState, title: title, style: {
                    marginBottom: shouldRenderChildren && hasChildren ? "10px" : "0"
                } }), shouldRenderChildren ? children : null] });
};
